"""
mbed SDK
Copyright (c) 2011-2013 ARM Limited

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from optparse import OptionParser

from workspace_tools.toolchains import TOOLCHAINS
from workspace_tools.targets import TARGET_NAMES


def get_default_options_parser():
    parser = OptionParser()
    
    parser.add_option("-m", "--mcu",
                  help="build for the given MCU (%s)" % ', '.join(TARGET_NAMES),
                  metavar="MCU")
    
    parser.add_option("-t", "--tool",
                  help="build using the given TOOLCHAIN (%s)" % ', '.join(TOOLCHAINS),
                  metavar="TOOLCHAIN")
    
    parser.add_option("-c", "--clean", action="store_true", default=False,
                  help="clean the build directory")
    
    parser.add_option("-o", "--options", action="append",
                  help='Add a build option ("save-asm": save the asm generated by the compiler, "debug-info": generate debugging information, "analyze": run static code analyzer")')
    
    return parser
